/* ex: set tabstop=2 expandtab: */
/*-
 * Copyright (c) 2010 by Bob Frazier and S.F.T. Inc. - all rights reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

//////////////////////////////////////////////////////////////////////////////
//                                 _                         _              //
//          _   _   __ _  _ __  __| | _   _  _ __    ___    | |__           //
//         | | | | / _` || '__|/ _` || | | || '_ \  / _ \   | '_ \          //
//         | |_| || (_| || |  | (_| || |_| || | | || (_) |_ | | | |         //
//          \__,_| \__,_||_|   \__,_| \__,_||_| |_| \___/(_)|_| |_|         //
//                                                                          //
//          Definitions for uarduno kernel driver for Arduino Uno           //
//                                                                          //
//  NOTE:  modified Allman (BSD) style, indent 2, and NO HARD TABS!         //
//         parameters in '()' indent with first '(' when practical          //
//         no space between '(' and things like 'if' 'for' 'while' fn       //
//         and 'return' does not need the extra '()' now does it?           //
//  if you simply must change the style before putting this in the kernel,  //
//  use the 'indent' utility with approrpiate switches.                     //
//////////////////////////////////////////////////////////////////////////////


// definitions used by this driver (common to all drivers)

#ifndef USB_DEBUG
#define USB_DEBUG 1 /* force debug for now; later remove */
#endif // USB_DEBUG

#define UARDUNO_PORT                     0 /* always specify port 0 */

#define UARDUNO_MODVER                   1      /* module version */

#define UARDUNO_CONFIG_INDEX             0
#define UARDUNO_IFACE_INDEX              0
#define UARDUNO_SECOND_IFACE_INDEX       1

#ifndef UARDUNO_INTR_INTERVAL
#define UARDUNO_INTR_INTERVAL            100     /* ms */
// for now this is a guess, based on UPLCOM; 0 would be default in 8.x
#endif

#define RSAQ_STATUS_CTS                  0x80   /* TODO:  get correct value */
#define RSAQ_STATUS_DSR                  0x02   /* TODO:  get correct value */
#define RSAQ_STATUS_DCD                  0x01   /* TODO:  get correct value */


struct uarduno_product
{
  uint16_t vendor;
  uint16_t product;
};




#if KERNELVER < 800000

/////////////////////////////////////////////////////////////////
//   _____                  ____  ____   ____    _____         //
//  |  ___|_ __  ___   ___ | __ )/ ___| |  _ \  |___  |__  __  //
//  | |_  | '__|/ _ \ / _ \|  _ \\___ \ | | | |    / / \ \/ /  //
//  |  _| | |  |  __/|  __/| |_) |___) || |_| |   / /_  >  <   //
//  |_|   |_|   \___| \___||____/|____/ |____/   /_/(_)/_/\_\  //
//                                                             //
/////////////////////////////////////////////////////////////////


struct uarduno_softc
{
  struct ucom_softc sc_ucom;
  //sub - classed from this driver, need its structure first

  int sc_iface_number;  /* interface number */

  usbd_interface_handle sc_intr_iface;  /* interrupt interface */
  int sc_intr_iface_number;   /* interrupt interface number */
  int sc_intr_number;   /* interrupt number */
  usbd_pipe_handle sc_intr_pipe;        /* interrupt pipe */
  u_char *sc_intr_buf;  /* interrupt buffer */
  int sc_isize;

  usb_cdc_line_state_t sc_line_state;   /* current line state */
  u_char sc_dtr;        /* current DTR state */
  u_char sc_rts;        /* current RTS state */
  u_char sc_status;

  u_char sc_lsr;        /* Local status register */
  u_char sc_msr;        /* uarduno status register - TODO:  is this applicable? */

  //int sc_chiptype;    /* Type of chip (was in uarduno, now deprecated) */

  struct task sc_task;
};


//internal prototypes

static usbd_status uarduno_reset(struct uarduno_softc *);
static usbd_status uarduno_set_line_coding(struct uarduno_softc *,
                                            usb_cdc_line_state_t *);
static usbd_status uarduno_set_crtscts(struct uarduno_softc *);
static void uarduno_intr(usbd_xfer_handle, usbd_private_handle, usbd_status);

static void uarduno_set(void *, int, int, int);
static void uarduno_dtr(struct uarduno_softc *, int);
static void uarduno_rts(struct uarduno_softc *, int);
static void uarduno_break(struct uarduno_softc *, int);
static usbd_status uarduno_set_line_state(struct uarduno_softc *);
static void uarduno_get_status(void *, int, u_char *, u_char *);
#if 0   /* TODO */
static int uarduno_ioctl(void *, int, u_long, caddr_t, int, usb_proc_ptr);
#endif
static int uarduno_param(void *, int, struct termios *);
static int uarduno_open(void *, int);
static void uarduno_close(void *, int);
static void uarduno_notify(void *, int);


#else // KERNELVER >= 800000

///////////////////////////////////////////////////////////////////
//   _____                  ____  ____   ____     ___            //
//  |  ___|_ __  ___   ___ | __ )/ ___| |  _ \   ( _ )   __  __  //
//  | |_  | '__|/ _ \ / _ \|  _ \\___ \ | | | |  / _ \   \ \/ /  //
//  |  _| | |  |  __/|  __/| |_) |___) || |_| | | (_) |_  >  <   //
//  |_|   |_|   \___| \___||____/|____/ |____/   \___/(_)/_/\_\  //
//                                                               //
///////////////////////////////////////////////////////////////////


//#define UARDUNO_BULK_BUF_SIZE 1024  /* bytes */ ORIGINAL 8.x version
#define UARDUNO_BULK_BUF_SIZE 0x40

// things defined in 7.x that 'went away'
#ifndef ISSET
#define ISSET(t, f)	((t) & (f))
#endif // ISSET 


enum
{
  UARDUNO_BULK_DT_WR,
  UARDUNO_BULK_DT_RD,
  UARDUNO_INTR_DT_RD,
  UARDUNO_N_TRANSFER,
};

struct uarduno_softc
{
  struct ucom_super_softc sc_super_ucom;
  struct ucom_softc sc_ucom;

  struct usb_xfer *sc_xfer[UARDUNO_N_TRANSFER];
  struct usb_device *sc_udev;
  struct mtx sc_mtx;

  struct usb_cdc_line_state sc_line_state;   /* current line state */

  uint16_t sc_line;

  uint8_t sc_lsr;         /* local status register */
  uint8_t sc_msr;         /* uarduno status register */
  uint8_t sc_ctrl_iface_no;
  uint8_t sc_data_iface_no;
  uint8_t sc_iface_index[2];
};

/* prototypes */

static usb_error_t uarduno_reset(struct uarduno_softc *, struct usb_device *);
static usb_error_t uarduno_set_line_state_reset(struct uarduno_softc *);
static void uarduno_cfg_set_dtr(struct ucom_softc *, uint8_t);
static void uarduno_cfg_set_rts(struct ucom_softc *, uint8_t);
static void uarduno_cfg_set_break(struct ucom_softc *, uint8_t);
static int  uarduno_pre_param(struct ucom_softc *, struct termios *);
static void uarduno_cfg_param(struct ucom_softc *, struct termios *);
static void uarduno_start_read(struct ucom_softc *);
static void uarduno_stop_read(struct ucom_softc *);
static void uarduno_start_write(struct ucom_softc *);
static void uarduno_stop_write(struct ucom_softc *);
static void uarduno_cfg_get_status(struct ucom_softc *, uint8_t *,
            uint8_t *);
static void uarduno_poll(struct ucom_softc *ucom);

static device_probe_t uarduno_probe;
static device_attach_t uarduno_attach;
static device_detach_t uarduno_detach;

static usb_callback_t uarduno_intr_callback;
static usb_callback_t uarduno_write_callback;
static usb_callback_t uarduno_read_callback;

static usb_error_t uarduno_set_line_coding(struct uarduno_softc *sc, struct usb_cdc_line_state * state);


#endif // KERNELVER >= 800000

/* end */


